close all;  clc; %clear all;
dos('taskkill  /IM cmd.exe');
tic;

%% Input
if(exist('u_g_in','var')==0)
    u_g_in=4;
end
if(exist('u_l_in','var')==0)
    u_l_in=4;
end

p_out = 50e5;
if(exist('minutes_to_simulate')==0) minutes_to_simulate=30;end
if(exist('minutes_to_leak')==0) minutes_to_leak=5;end
if(exist('leak_magnitude','var')==0) leak_magnitude=0.15;end
if(exist('shutdown','var')==0) shutdown=0;end

temperature=4;
inches=20; %Pipe diameter in inches
slength=100;
if(exist('lsegment','var')==0)
    lsegment=16; %hvilket segment i olga er lekkasjen p
end
leak_pos=(lsegment-5)*slength-slength/2;
dt=0.02;

% Initialization
u_g=u_g_in;
u_l=u_l_in;
temperature_in = temperature; %temp of mass going in
temperature_out = temperature; %temp of mass going out
D=inches*2.54e-2;
Area=pi*D^2/4;

%% Initialize OLGA
try
    % Initializing the OLGA server and loading file
    for tries=1:100
    [errmsg]=dos('olga-5.2.1.exe -server olga_1 &');
    [errmsg]=OLGAConnect('localhost olga_1');
        if isempty(errmsg)
            disp(['Olga started'])
            break
        else
            disp(['Connection failed, new attempt:'])
            pause(10);
        end
    end
    if(exist('var_model','var')==0)
        [errmsg]=OLGAReadInput(['OLGA_filer\model_twophase.geninp']);
    else
        [errmsg]=OLGAReadInput(var_model);
    end
    
    % Giving input variables
    [errmsg]=OLGASetInputVar('$SOURCE-1 SOURCE $NODE_1 BOUNDARY $CONTROLLER-01 CONTROLLER $SOURCE-2 SOURCE $SOURCE-3 SOURCE',[5 5 1 5 5]);
catch
    dos('taskkill  /IM cmd.exe');
    rethrow(lasterror);
end

[errmsg]=OLGASetTrendVar(['$INLET PT AL $OUTLET UG QLTHL QG'...
    ' $SOURCE-1 GGSOUR GLHLMA $OUTLET GLTHL GG $INLET-2 UG UL $LEAK-01 GGLEAK GLHLLK GTLEAK $INLET UG $INLET-VELOCITY UG UL']);

%% Find initial values
w_g=u_g*50*0.5*Area; %setter steady state flow basert p u_g og u_l
w_l=u_l*900*(0.5)*Area;
    
input_olga=[w_g/(w_g+w_l) w_g+w_l -1 temperature_in 0 ...
    1 -1 p_out temperature_out 0 0 ...
    1 0 -1 temperature_in 0 ...
    0 0 -1 temperature_in 0];

[errmsg]=OLGASetInputData(input_olga);
[errmsg]=OLGAInitialize;
[t0,d,errmsg] = OLGASendProfileData('$FLOWPATH_0 ZZBOU');
Nnodes=length(d.Data)-10;

%% Simulation parameters
T_end=60*minutes_to_simulate; % 60 seconds * x minutes
delay=minutes_to_leak;
[errmsg]=OLGASetTend(T_end);
simlength=T_end/dt;
samp_time=T_end/2000; %sampling time for plotting (in seconds)
samples=samp_time/dt; 
ll=0;

%% Initialize arrays
t=zeros(simlength,1);
p_mod_L_store=zeros(simlength,1);
p_mod_0_store=zeros(simlength,1);
u_g_mod_0_store=zeros(simlength,1);
u_g_mod_L_store=zeros(simlength,1);
u_l_mod_0_store=zeros(simlength,1);
u_l_mod_L_store=zeros(simlength,1);
epsilon_mod_0_store=zeros(simlength,1);
epsilon_mod_L_store=zeros(simlength,1);
rho_g_mod_0_store=zeros(simlength,1);
rho_l_mod_0_store=zeros(simlength,1);
rho_g_mod_L_store=zeros(simlength,1);
rho_l_mod_L_store=zeros(simlength,1);
temp_mod_0_store=zeros(simlength,1);
gmf_mod_L=zeros(simlength,1);

w_g_0_mod_store=zeros(simlength,1);
w_l_0_mod_store=zeros(simlength,1);
w_g_L_mod_store=zeros(simlength,1);
w_l_L_mod_store=zeros(simlength,1);
m_g_0_mod_store=zeros(simlength,1);
m_l_0_mod_store=zeros(simlength,1);
m_g_L_mod_store=zeros(simlength,1);
m_l_L_mod_store=zeros(simlength,1);

leak_mod_g=zeros(simlength,1);
leak_mod_l=zeros(simlength,1);
leak_mod_total=zeros(simlength,1);

p_mod_surf=zeros(floor(simlength/samples), Nnodes-1);
u_g_mod_surf=zeros(floor(simlength/samples), Nnodes);
u_l_mod_surf=zeros(floor(simlength/samples), Nnodes);
epsilon_mod_surf=zeros(floor(simlength/samples), Nnodes-1);
slug_flow=zeros(1,90000);
format long

%% Running simulations
try
for ii=1:simlength

    [t0, y0, p0, errmsg] = OLGASimStep(ii*dt, input_olga);
    t(ii)=t0;

    % Sinusoidal:
%     u_g=u_g_in;%+u_g_in*0.2*abs(round(sin(ii*6*pi/simlength)));
%     u_l=u_l_in-u_l_in*0.2*abs(round(sin(ii*6*pi/simlength)));
%     u_g=u_g_in+u_g_in*0.2*sin(ii*6*pi/simlength);
%     u_l=u_l_in-u_l_in*0.2*sin(ii*6*pi/simlength);

    if shutdown==1
        if ii<7*60/dt
            slug_flow(ii)=1;
        elseif ii<8.5*60/dt
            slug_flow(ii)=1-0.4*(ii/60*dt-7)/1.5;
        else
            slug_flow(ii)=0.6;
        end
        w_g=u_g*50*0.5*Area;
        w_l=u_l*900*(0.5)*Area;
        input_olga(1)=w_g/(w_g+w_l);
        input_olga(2)=slug_flow(ii)*(w_g+w_l);
    end

    % finding pressure and density at each section and extrapolating values
    [tt,d,errmsg] = OLGASendProfileData('$FLOWPATH_0 PT UG UL AL ROG ROL GG GLTHL');
    %PRESSURE
    p_mod_0_store(ii)=extrap_inlet(d(1).Data(3:5));
    p_mod_L_store(ii)=extrap_outlet(d(1).Data(end-2:end));
    %VELOCITIES

    u_g_mod_0_store(ii)=d(2).Data(3);
    u_g_mod_L_store(ii)=d(2).Data(end);
    u_l_mod_0_store(ii)=d(3).Data(3);
    u_l_mod_L_store(ii)=d(3).Data(end);
    %FRACTION
    epsilon_mod_0_store(ii)=extrap_inlet(d(4).Data(3:5));
    epsilon_mod_L_store(ii)=extrap_outlet(d(4).Data(end-2:end));

    m_g_0_mod_store(ii)=d(7).Data(3)/Area/d(2).Data(3);%extrap_inlet(d(5).Data(3:5))*epsilon_mod_0_store(ii);
    m_l_0_mod_store(ii)=d(8).Data(3)/Area/d(3).Data(3);%extrap_inlet(d(6).Data(3:5))*(1-epsilon_mod_0_store(ii));
    m_g_L_mod_store(ii)=d(7).Data(end)/Area/d(2).Data(end);%extrap_outlet(d(5).Data(end-2:end))*epsilon_mod_L_store(ii);
    m_l_L_mod_store(ii)=d(8).Data(end)/Area/d(3).Data(end);%extrap_outlet(d(6).Data(end-2:end))*(1-epsilon_mod_L_store(ii));
    w_g_0_mod_store(ii)=d(7).Data(3)/Area;%m_g_0_mod_store(ii)*u_g_mod_0_store(ii);
    w_l_0_mod_store(ii)=d(8).Data(3)/Area;%m_l_0_mod_store(ii)*u_l_mod_0_store(ii);
    w_g_L_mod_store(ii)=d(7).Data(end)/Area;%m_g_L_mod_store(ii)*u_g_mod_L_store(ii);
    w_l_L_mod_store(ii)=d(8).Data(end)/Area;%m_l_L_mod_store(ii)*u_l_mod_L_store(ii);

    %if leak is two negative source or one positive
    if input_olga(13)~=0 || input_olga(18)~=0
        leak_mod_g(ii)=input_olga(13);%y0(12).Data;
        leak_mod_l(ii)=input_olga(18);%y0(13).Data;
    else
        leak_mod_g(ii)=y0(12).Data;
        leak_mod_l(ii)=y0(13).Data;
        leak_mod_total(ii)=y0(14).Data;
    end
    if ii*dt>=minutes_to_leak*60
        if leak_magnitude ~= -1
        input_olga(11)=leak_magnitude;
        else
          input_olga(13)=-d(7).Data(lsegment-1)*Area*0.2;
          input_olga(18)=-d(8).Data(lsegment-1)*Area*0.2;
        end
    end
    if(mod(ii,samples)==0 || ii==1)
        [tt,d,errmsg] = OLGASendProfileData('$FLOWPATH_0 PT UG ROG ROL UL AL TM GASMFR GG GL ID');
        pause(0.02);
        ll=round(ii/samples+1);
        u_g_mod_surf(ll,:)=d(2).Data(7:end-4);
        u_l_mod_surf(ll,:)=d(5).Data(7:end-4);
        p_mod_surf(ll,:)=d(1).Data(7:end-4);
        epsilon_mod_surf(ll,:)=d(6).Data(7:end-4);
    end
    if(mod(ii,samples*5)==0)
        disp(['Progress: ', num2str(ii/simlength*100), ' %'])
        disp(sprintf('Flow regime inlet:%d center:%d outlet:%d ',d(11).Data(5),d(11).Data(round(length(d(11).Data)/2)),d(11).Data(end)))
        disp(['Inlet flow:   ',num2str(d(9).Data(3),3),' gas   -   ',num2str(d(10).Data(3),3),' oil']);
        disp(['Lekkasje: ', num2str(leak_mod_g(ii)),'kg/s'])
    end
end
catch
    dos('taskkill  /IM cmd.exe');
    rethrow(lasterror);
end
%% stopping server

% stopping the server and disconnecting
[errmsg]=OLGAStopServer;
[errmsg]=OLGADisconnect;
dos('taskkill  /IM cmd.exe');
%%
load('mat_files\Model_tracker');

Mversjon=Mversjon+1;
dato{Mversjon}=clock;
save('mat_files\Model_tracker','Mversjon','dato','-APPEND');

CLOCK = dato{Mversjon};
time_string=[num2str(CLOCK(2)),num2str(CLOCK(3)),num2str(CLOCK(4)),num2str(CLOCK(5))];
model_string=['mat_files/modeldata_v',num2str(Mversjon),'_Date-',time_string,'_SimLength-',num2str(T_end/60),'_Leakpos-',num2str(leak_pos)];
save(model_string,'dt','t','T_end','*_mod_*','samp_time','Area','Nnodes','leak_pos','u_g_in','u_l_in','Mversjon','slength');
save('last','model_string');

toc;
