% function run_basic_observer_twophase2(in,in2,in3)
% clear all;
close all; clc; tic;
dos('taskkill  /IM cmd.exe');
format long;format compact;
%%
load('mat_files\Observer_tracker');
load('mat_files\Model_tracker');
versjon=versjon+1;
dato{versjon}=clock;
save('mat_files\Observer_tracker','versjon','dato');
%% Input

if(exist('error','var')==0) error=0; end
if(exist('in','var')==0) in=1; end
if(exist('in2','var')==0) in2=0; end
if(exist('in3','var')==0) in3=0; end
if(exist('restart_interval','var')==0) restart_interval=30; end
if(exist('use_correct_epsilon_l','var')==0) use_correct_epsilon_l=0; end
if(exist('start_leak_adaption','var')==0) start_leak_adaption=5; end
if(exist('leak_gas_fraction','var')==0) leak_gas_fraction=-1; end
if(exist('leak_adaption_magnitude','var')==0) leak_adaption_magnitude=0.00004; end
if(exist('leak_adaption_position','var')==0) leak_adaption_position=0.2; end

dooi=0.0; %degree_of_output_injection
output_injection=in; % 1 if output injection on
output_injection_delay=15; %seconds
output_injection_outlet_off=in2; % 1 if no output injection at outlet
temperature = 4;



c_G=314.5;
c_L=1161.3;
k_l=1.211339165862600e+009;

const=struct(...
    'c_G',c_G,'c_L',c_L,...
    'k_l',k_l, ...
    'k_x',0,'k_u',0,...
    'dooi',dooi,...
    'output_injection',output_injection,...
    'oi',0);


load('last');
load(model_string);

%% Initializing the OLGA server and loading file
for tries=1:100
[errmsg]=dos('olga-5.2.1.exe -server olga_1 &');
[errmsg]=OLGAConnect('localhost olga_1');
    if isempty(errmsg)
        disp(['Olga started'])
        break
    else
        disp(['Connection failed, new attempt:'])
        pause(10);
    end
end
OLGAReadInput(['OLGA_filer\observer_twophase.geninp']);
OLGAInitialize;
try
    [t0,d,errmsg] = OLGASendProfileData('$FLOWPATH_0 ZZBOU');
catch
    dos('taskkill  /IM cmd.exe');
    rethrow(lasterror);
end
%% Initializing the observer struct
obs=struct();
obs.area=Area;
obs.amb=101300;
obs.leakpos=3012.5;
obs.u_s=0;
obs.frac=0.1;
L=slength*50;
vindu=60/dt;

obs.x=d(1).Data;
obs.Nnodes=length(obs.x)-1;
obs.dx=obs.x(2:end)-obs.x(1:end-1);
obs.x=obs.x(1:end-1)+obs.dx/2;
obs.dx=[obs.x(1)-d(1).Data(1) obs.x(2:end)-obs.x(1:end-1)];

%% making array with InputVars

Ncontrollers=54;
sources='';
for kk=3:Ncontrollers
    sources=[sources ' ' '$SOURCE-' num2str(kk) ' SOURCE'];
end
input_source=repmat([-1 0 -1 temperature 0],1,Ncontrollers-2);
Ninputs=ones(1,(Ncontrollers-2))*5;

%% Giving input variables
OLGASetInputVar('$SOURCE-1 SOURCE $SOURCE-2 SOURCE $NODE_1 BOUNDARY',[5 5 5]);
OLGASetInputVar(sources,Ninputs,1);
OLGASetTrendVar(['$INLET ROG ROL $OUTLET UG QLTHL QG'...
    ' $SOURCE-1 GTSOUR $SOURCE-2 GLHLMA $OUTLET GLTHL GG $INLET-2 UG UL']);
w_g=u_g_in*(1+error)*50*0.5*Area;
w_l=u_l_in*(1+error)*900*(0.5)*Area;
input_olga=[w_g/(w_g+w_l) w_g+w_l -1 temperature 0 ...
    0 0 -1 temperature 0 ...
    1 -1 p_mod_L_store(1)*(1+error*in3/10) temperature 0 ...
    input_source];

OLGASetInputData(input_olga);
OLGAInitialize;

input_olga(1)=1;
input_olga(6)=0;

OLGASetTend(T_end);
simlength=T_end/dt;
samples=samp_time/dt;
ll=0;

leak_adaption_started=0;
restarted=0;
%%

p_obs_L_store=zeros(simlength,1);
p_obs_0_store=zeros(simlength,1);
u_g_obs_0_store=zeros(simlength,1);
u_g_obs_L_store=zeros(simlength,1);
u_l_obs_0_store=zeros(simlength,1);
u_l_obs_L_store=zeros(simlength,1);
epsilon_obs_0_store=zeros(simlength,1);
epsilon_obs_L_store=zeros(simlength,1);

w_g_obs_0_store_filtered=zeros(simlength,1);
w_l_obs_0_store_filtered=zeros(simlength,1);
w_g_obs_0_store_unfiltered=zeros(simlength,1);
w_l_obs_0_store_unfiltered=zeros(simlength,1);

% Generate error on measurements
if(exist('w_g_err','var')==1) w_g_error=w_g_err;
else w_g_error=0; end
if(exist('w_l_err','var')==1) w_l_error=w_l_err;
else w_l_error=0; end
if(exist('p_err','var')==1) p_error=p_err;
else p_error=0; end
if(exist('epsilon_err','var')==1) epsilon_error=epsilon_err;
else epsilon_error=0; end

w_g_0_mod_store=w_g_0_mod_store*(1+w_g_error);
w_l_0_mod_store=w_l_0_mod_store*(1+w_l_error);
w_g_L_mod_store=w_g_L_mod_store*(1+w_g_error);
w_l_L_mod_store=w_l_L_mod_store*(1+w_l_error);
p_mod_L_store=p_mod_L_store+(ones(size(p_mod_L_store))*p_error);
p_mod_0_store=p_mod_0_store+(ones(size(p_mod_0_store))*p_error);
epsilon_mod_0_store=epsilon_mod_0_store+(ones(size(epsilon_mod_0_store))*epsilon_error);
epsilon_mod_L_store=epsilon_mod_L_store+(ones(size(epsilon_mod_L_store))*epsilon_error);

w_g_0_obs_store=zeros(simlength,1);
w_l_0_obs_store=zeros(simlength,1);
w_g_L_obs_store=zeros(simlength,1);
w_l_L_obs_store=zeros(simlength,1);
m_g_0_obs_store=zeros(simlength,1);
m_l_0_obs_store=zeros(simlength,1);
m_g_L_obs_store=zeros(simlength,1);
m_l_L_obs_store=zeros(simlength,1);

p_obs_surf=zeros(floor(simlength/samples), Nnodes-1);
u_g_obs_surf=zeros(floor(simlength/samples), Nnodes);
u_l_obs_surf=zeros(floor(simlength/samples), Nnodes);
epsilon_obs_surf=zeros(floor(simlength/samples), Nnodes-1);

leakpos_obs_store=zeros(simlength,1);
u_s_obs_store=zeros(simlength,1);
obs_leak_g=zeros(simlength,1);
obs_leak_l=zeros(simlength,1);
obs_leak_total=zeros(simlength,1);

obs_q_mod_0_store=zeros(simlength,4);
obs_q_mod_L_store=zeros(simlength,4);
obs_q_obs_0_store=zeros(simlength,4);
obs_q_obs_L_store=zeros(simlength,4);
obs_Lambda_0_store=zeros(simlength,4);
obs_Lambda_L_store=zeros(simlength,4);

p_tilde_0=zeros(simlength,1);
obs_mG_tilde_L_store=zeros(simlength,1);
obs_mL_tilde_L_store=zeros(simlength,1);

ii_select_obs=zeros(simlength,1);

%% Load previous steady state
if restart_files(Mversjon)==3 && w_g_error==0 && w_l_error==0 && p_error==0 && epsilon_error==0
   OLGALoadSnap(['leak_start',num2str(Mversjon)]);
   load(['mat_files\pre_leak_',num2str(Mversjon)]);
    const.k_x=leak_adaption_position;
    const.k_u=leak_adaption_magnitude;
    leak_adaption_started=1;
   
    disp('Loaded previously obtained steady state');
else
    ii_start=1;
end
disp(['Pre-simulation time ',num2str(toc)]);
last_restart=0;
tic;
%% Simulate
for ii=ii_start:simlength
    if leak_adaption_started==1
        %Check if it is time to restart
        if(((L-obs.leakpos)<max(mean(u_g_obs_L_store(ii-floor(vindu):ii)),mean(u_l_obs_L_store(ii-floor(vindu):ii)))*(ii-last_restart)*dt)) && leak_adaption_started==1 % || w_g_0_mod_store(ii_select_obs(ii-1))< -100*0.1*w_g_0_mod_store(1) || w_l_0_mod_store(ii_select_obs(ii-1))< -100*0.1*w_l_0_mod_store(1) || ii_select_obs(ii-1)>7*60/dt) && leak_adaption_started==1
           diary(['mat_files\restart_',num2str(versjon),'.out']);
           disp(['Leakpos=',num2str(obs.leakpos)]);
           disp(['Velocity=',num2str(max(mean(u_g_obs_L_store(ii-floor(vindu):ii)),mean(u_l_obs_L_store(ii-floor(vindu):ii))))]);       
           disp(['Restarting at time',num2str(ii_select_obs(ii-1)*dt)]);
           diary off;
           disp('loading snapfile')
           OLGALoadSnap(['leak_start',num2str(Mversjon)]);
           last_restart=ii;
        end
    end
    % Check if it is time to start leak adaption
    if ii*dt>=start_leak_adaption*60 && leak_adaption_started==0
       if restart_files(Mversjon)==0
            disp('saving snapfile')
            OLGASaveSnap(['leak_start',num2str(Mversjon)]);
            ii_start=ii;
            save(['mat_files\pre_leak_',num2str(Mversjon)],'*obs*','*tilde*','ii_start');
            restart_files(Mversjon)=1;
            save('mat_files\Model_tracker','restart_files','-APPEND');
            const.k_x=leak_adaption_position;
            const.k_u=leak_adaption_magnitude;
            leak_adaption_started=1;
            last_restart=ii;
       elseif w_g_error~=0 || w_l_error~=0 || p_error~=0 || epsilon_error~=0
            const.k_x=leak_adaption_position;
            const.k_u=leak_adaption_magnitude;
            leak_adaption_started=1;
            last_restart=ii;
       else
           'Skal ikke skje'
            const.k_x=leak_adaption_position;
            const.k_u=leak_adaption_magnitude;
            leak_adaption_started=1;
            last_restart=ii;
       end
    end
    
    if ii*dt>=output_injection_delay && const.oi==0
        'P med oi'
        const.oi=1;
    end
     ii_select_obs(ii)=leak_adaption_started*start_leak_adaption*60/dt+(ii-last_restart);% ii-round(restarted*restart_interval*60/dt);
    
    [t0, y0, trash, errmsg] = OLGASimStep(t(ii_select_obs(ii)), input_olga);
    
    [tt,d,errmsg] = OLGASendProfileData('$FLOWPATH_0 PT UG UL AL ROG ROL GG GLTHL');

    p_obs_0_store(ii)=extrap_inlet(d(1).Data(3:5));
    p_obs_L_store(ii)=extrap_outlet(d(1).Data(end-2:end));
    u_g_obs_0_store(ii)=d(2).Data(3);
    u_g_obs_L_store(ii)=d(2).Data(end);
    u_l_obs_0_store(ii)=d(3).Data(3);
    u_l_obs_L_store(ii)=d(3).Data(end);
    epsilon_obs_0_store(ii)=extrap_inlet(d(4).Data(3:5));
    epsilon_obs_L_store(ii)=extrap_outlet(d(4).Data(end-2:end));
    
    m_g_0_obs_store(ii)=d(7).Data(3)/Area/d(2).Data(3);
    m_l_0_obs_store(ii)=d(8).Data(3)/Area/d(3).Data(3);
    m_g_L_obs_store(ii)=d(7).Data(end)/Area/d(2).Data(end);
    m_l_L_obs_store(ii)=d(8).Data(end)/Area/d(3).Data(end);
    w_g_0_obs_store(ii)=d(7).Data(3)/Area;
    w_l_0_obs_store(ii)=d(8).Data(3)/Area;
    w_g_L_obs_store(ii)=d(7).Data(end)/Area;
    w_l_L_obs_store(ii)=d(8).Data(end)/Area;
    
    %Find boundary values
   [w P q_0 q_L q_hat_0 q_hat_L Lambda_0 Lambda_L P_Delta er w_er]=find_boundaries_numerical(u_g_obs_L_store(ii),u_g_obs_0_store(ii),u_g_mod_L_store(ii_select_obs(ii)),u_g_mod_0_store(ii_select_obs(ii)), ...
                            u_l_obs_L_store(ii),u_l_obs_0_store(ii),u_l_mod_0_store(ii_select_obs(ii)),u_l_mod_L_store(ii_select_obs(ii)), ...
                            epsilon_mod_0_store(ii_select_obs(ii)),epsilon_obs_0_store(ii),epsilon_mod_L_store(ii_select_obs(ii)), ...
                            p_mod_0_store(ii_select_obs(ii)),p_obs_0_store(ii),p_mod_L_store(ii_select_obs(ii)),const,...
                            m_g_0_obs_store(ii),m_l_0_obs_store(ii),w_g_L_obs_store(ii),w_l_L_obs_store(ii),...
                            m_g_0_mod_store(ii_select_obs(ii)),m_l_0_mod_store(ii_select_obs(ii)),w_g_0_mod_store(ii_select_obs(ii)),w_l_0_mod_store(ii_select_obs(ii)),...
                            m_g_L_mod_store(ii_select_obs(ii)),m_l_L_mod_store(ii_select_obs(ii)),w_g_L_mod_store(ii_select_obs(ii)),w_l_L_mod_store(ii_select_obs(ii)));
    
    obs_q_mod_0_store(ii,:)=q_0';
    obs_q_mod_L_store(ii,:)=q_L';
    obs_q_obs_0_store(ii,:)=q_hat_0';
    obs_q_obs_L_store(ii,:)=q_hat_L';
    obs_Lambda_0_store(ii,:)=Lambda_0';
    obs_Lambda_L_store(ii,:)=Lambda_L';
    

    obs_mG_tilde_L_store(ii)=w_er(1);
    obs_mL_tilde_L_store(ii)=w_er(2);
  
    if ii==1
        w_g_obs_0_store_filtered(ii)=w(1);
        w_l_obs_0_store_filtered(ii)=w(2);
        p_tilde_0(ii)=P_Delta;
    else
        p_tilde_0(ii)=p_tilde_0(ii-1)-dt/1*(p_tilde_0(ii)-P_Delta);
        T=min(6.25*2/max(min(abs(u_g_obs_0_store(ii)),abs(u_l_obs_0_store(ii))),0.1),300);
        w_g_obs_0_store_filtered(ii)=w_g_obs_0_store_filtered(ii-1)-dt/T*(w_g_obs_0_store_filtered(ii-1)-w(1));
        w_l_obs_0_store_filtered(ii)=w_l_obs_0_store_filtered(ii-1)-dt/T*(w_l_obs_0_store_filtered(ii-1)-w(2));
        
    end
    w_g_obs_0_store_unfiltered(ii)=w(1);
    w_l_obs_0_store_unfiltered(ii)=w(2); 
    
    w_g_obs_0_store_filtered(ii)=max(min(1000,w_g_obs_0_store_filtered(ii)),0);
    w_l_obs_0_store_filtered(ii)=max(min(14000,w_l_obs_0_store_filtered(ii)),0);
    
    w_g=w_g_obs_0_store_filtered(ii);
    w_l=w_l_obs_0_store_filtered(ii);
    input_olga(2)=w_g*Area;
    input_olga(7)=w_l*Area;
    input_olga(13)=p_mod_L_store(ii_select_obs(ii))+P*(1-output_injection_outlet_off);
    p_obs_L_store(ii)=p_mod_L_store(ii_select_obs(ii))+P*(1-output_injection_outlet_off);
    
    %Leak adaption
    [sec_1, sec_2, sec_1_frac, sec_2_frac,...
    sec_1_mass, sec_2_mass, obs.leakpos, obs.u_s, obs.frac...
        ]=find_leak_num2(obs,er,d(1).Data,d(4).Data,d(2).Data,d(5).Data,d(3).Data,d(6).Data,leak_gas_fraction,obs_q_mod_0_store(ii,4)-obs_q_obs_0_store(ii,4),w_er, dt,const,use_correct_epsilon_l,leak_mod_g(ii_select_obs(ii))/(leak_mod_g(ii_select_obs(ii))+leak_mod_l(ii_select_obs(ii))));    
    input_olga=[input_olga(1:15) input_source];
    
    input_olga(15+(sec_1-3)*5+1)=sec_1_frac;
    input_olga(15+(sec_1-3)*5+2)=-sec_1_mass;
    input_olga(15+(sec_2-3)*5+1)=sec_2_frac;
    input_olga(15+(sec_2-3)*5+2)=-sec_2_mass;

    leakpos_obs_store(ii)=obs.leakpos-12.5;
    u_s_obs_store(ii)=obs.u_s;

    if sec_1_frac<0
        obs_leak_g(ii)=0;
        obs_leak_l(ii)=0;
    else
        obs_leak_g(ii)=-sec_1_frac*sec_1_mass-sec_2_frac*sec_2_mass;
        obs_leak_l(ii)=-(1-sec_1_frac)*sec_1_mass-(1-sec_2_frac)*sec_2_mass;
    end

    if(mod(ii,samples)==0 || ii==1)
        [tt,d,errmsg] = OLGASendProfileData('$FLOWPATH_0 PT UG ROG ROL UL AL TM QLTHL QG GASMFR ID');
        pause(0.02);
        ll=round(ii/samples+1);
        u_g_obs_surf(ll,:)=d(2).Data(7:end-4);
        u_l_obs_surf(ll,:)=d(5).Data(7:end-4);
        p_obs_surf(ll,:)=d(1).Data(7:end-4);
        epsilon_obs_surf(ll,:)=d(6).Data(7:end-4);
        disp(['mG_hat/mL_hat: ',num2str(m_g_0_obs_store(ii),4),'/',num2str(m_l_0_obs_store(ii),4),'                           mG/mL: ',num2str(m_g_0_mod_store(ii),4),'/',num2str(m_l_0_mod_store(ii),4)])
        disp(['wG_hat/wL_hat: ',num2str(w(1),4),'/',num2str(w(2),4),'  wG/wL_hat_filt: ',num2str(w_g,4),'/',num2str(w_l,4),'  wG/wL: ',num2str(w_g_0_mod_store(ii),4),'/',num2str(w_l_0_mod_store(ii),4)])
        disp(['Leakpos: ',num2str(leakpos_obs_store(ii),4),'(',num2str(leak_pos,4),')  Leakage gas: ',num2str(obs_leak_g(ii),4),'(',num2str(leak_mod_g(ii_select_obs(ii))),')  oil: ',num2str(obs_leak_l(ii),4),'(',num2str(leak_mod_l(ii_select_obs(ii))),')'])
        if leak_adaption_started==1
            disp([num2str((L-obs.leakpos)),' is more than ',num2str(max(mean(u_g_obs_L_store(ii-floor(vindu):ii)),mean(u_l_obs_L_store(ii-floor(vindu):ii)))*(ii-last_restart)*dt)]);
        end
    end
    if(mod(ii,samples*10)==0)
        disp(['Progress: ', num2str(ii/simlength*100), ' %'])
%         disp(['Fraction: ',num2str(obs.frac)])
        disp(sprintf('Flow regime inlet:%d center:%d outlet:%d ',d(11).Data(1),d(11).Data(round(length(d(11).Data)/2)),d(11).Data(end)))
%         disp(['Position: ', num2str(leakpos_obs_store(ii))])
    end
end
%%
[errmsg]=OLGAStopServer;
[errmsg]=OLGADisconnect;
dos('taskkill  /IM cmd.exe');

toc;


plot_olga_observer;
obs_string=[model_string,'_obs',num2str(versn)];
save([model_string,'_obs',num2str(versn)],'*surf','*obs*','*error');
